/**
 * GSL based multivariate gaussian generator 
 * Copyright Emmanuel Benazera, 2003.
 */

#ifndef _GSL_MGAUSS_H
#define _GSL_MGAUSS_H

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_permutation.h>

#undef __BEGIN_DECLS
#undef __END_DECLS
#ifdef __cplusplus
# define __BEGIN_DECLS extern "C" {
# define __END_DECLS }
#else
# define __BEGIN_DECLS /* empty */
# define __END_DECLS /* empty */
#endif

__BEGIN_DECLS

void gsl_ran_multivariate_gaussian (const gsl_rng *r,
				    gsl_vector *mu,
				    gsl_vector *eval, gsl_matrix *evec,
				    gsl_vector *workspace, gsl_vector *px);

void gsl_ran_multivariate_gaussian_chol_decomp (const gsl_rng *r,
						gsl_vector *mu,
						gsl_matrix *S,
						gsl_vector *px);

void gsl_ran_multivariate_gaussian_chol_no_decomp (const gsl_rng *r,
						   gsl_vector *mu,
						   gsl_matrix *L,
						   gsl_vector *px);

void gsl_ran_multivariate_gaussian_pdf_LU (gsl_vector *x,
					   gsl_vector *mu, gsl_matrix *LU_sigma,
					   gsl_permutation *p, int signum,
					   gsl_vector *wks_v, gsl_matrix *wks_m, 
					   double *det, double *r);

void gsl_ran_multivariate_gaussian_pdf (gsl_vector *x,
					gsl_vector *mu, gsl_matrix *inv,
					double det, gsl_vector *workspace,
					double *r);

void gsl_ran_multivariate_gaussian_ratio_method (const gsl_rng *r,
						 gsl_vector *mu,
						 gsl_vector *eval, gsl_matrix *evec,
						 gsl_vector *workspace, gsl_vector *px);

__END_DECLS
  
#endif
